package it.arquillian.moduleTest.jpa.controller;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import it.arquillian.moduleTest.jpa.ArchiveFactory;
import it.arquillian.moduleTest.jpa.entity.Action;
import it.arquillian.moduleTest.jpa.util.ManageExceptionStackTrace;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.ejb.EJB;
import javax.inject.Inject;
import javax.transaction.UserTransaction;

import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.persistence.Cleanup;
import org.jboss.arquillian.persistence.CleanupStrategy;
import org.jboss.arquillian.persistence.ShouldMatchDataSet;
import org.jboss.arquillian.persistence.TestExecutionPhase;
import org.jboss.arquillian.persistence.TransactionMode;
import org.jboss.arquillian.persistence.Transactional;
import org.jboss.arquillian.persistence.UsingDataSet;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

/**
 * Test ActionJpaController EJB
 *
 * @author Marco!Cella
 */
@RunWith(Arquillian.class)
public class ActionJpaControllerTest extends ArchiveFactory {

    @EJB
    private ActionJpaController actionJpaController;

    @Inject
    private UserTransaction transaction;

    private static String className = ActionJpaControllerTest.class.getName();

    // How to handle "Caught exception in beforeCompletion()" exception ?
    // Database table is not cleaned after test execution
    @Test
    @UsingDataSet("jpa/controller/ActionJpaControllerTest/ShouldNotEditAction.xml")
    @ShouldMatchDataSet(value = "jpa/controller/ActionJpaControllerTest/ShouldNotEditActionAssert.xml")
    @Cleanup(phase = TestExecutionPhase.AFTER, strategy = CleanupStrategy.USED_ROWS_ONLY)
    @Transactional(value = TransactionMode.DISABLED)
    public void shouldNotEditActionOnNullNameField() throws Exception {
        transaction.begin();

        Logger.getLogger(className).log(Level.INFO, "\nExecute test: {0} [#] shouldNotEditActionOnNullNameField", className);
        // given
        Action givenAction = actionJpaController.findAction( Integer.valueOf(3) );
        assertNotNull( "Action entity must not be null:\n", givenAction );
        Class expectedExceptionClass = javax.validation.ConstraintViolationException.class;
        boolean testStatus = false;
        String actualException = "No exception";
        try {
            // when
            givenAction.setName( null );
            actionJpaController.edit( givenAction );
            transaction.commit();
            // then
        } catch(Throwable ex){
            testStatus = ManageExceptionStackTrace.existInnerException(ex, expectedExceptionClass);
            if(!testStatus) { actualException = ManageExceptionStackTrace.getInnerException(ex); }
        }
        assertFalse( "Should have throws javax.validation.ConstraintViolationException: validation failed for classes [it.arquillian.moduleTest.jpa.entity.Action] during update time for groups [javax.validation.groups.Default, ].\n" + actualException + " was thown instead.", testStatus );
    }

    @Before
    public void before() {
       System.out.println("Before BLA");
    }

    @After
    public void after() {
       System.out.println("After BLA");
    }
}
